#Copyright (c) 2015 3D Control Systems LTD

#3DPrinterOS client is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#3DPrinterOS client is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with 3DPrinterOS client.  If not, see <http://www.gnu.org/licenses/>.

# Author: Oleg Panasevych <panasevychol@gmail.com>, Vladimir Avdeev <another.vic@yandex.ru>

import os
import sys
import logging
from subprocess import Popen, PIPE

import config
from checker_waiter import CheckerWaiter

def is_admin():
    import ctypes, os
    try:
        is_admin = os.getuid() == 0
    except:
        is_admin = ctypes.windll.shell32.IsUserAnAdmin()
    return is_admin

def launch_suprocess(file_name):
    logger = logging.getLogger(__name__)
    logger.info('Launching subprocess ' + file_name)
    client_dir = os.path.dirname(os.path.abspath(__file__))
    path = os.path.join(client_dir, file_name)
    try:
        process = Popen([sys.executable, path])
    except Exception as e:
        logger.warning('Could not launch ' + file_name + ' as subprocess due to error:\n' + e.message)
    else:
        return process

class RightsCheckerWaiter(CheckerWaiter):

    NAME_FOR_LOGGING = 'Linux groups membership'

    def __init__(self, app):
        CheckerWaiter.__init__(self, app, self.check_if_user_in_groups)

    def check_if_user_in_groups(self):
        if sys.platform.startswith('linux') and config.get_settings()['linux_rights_warning'] and not is_admin():
            self.logger.debug('Checking Linux rights')
            result = self.execute_command('groups')
            if not ('tty' in result and 'dialout' in result and 'usbusers' in result):
                self.logger.debug('Current Linux user is not in tty and dialout groups')
                return False
        return True

    def add_user_groups(self):
        if sys.platform.startswith('linux'):
            self.logger.debug('Adding Linux user to necessary groups')
            self.execute_command(['groupadd', 'usbusers'])
            self.execute_command('gksu "sudo usermod -a -G dialout,tty,usbusers $USER"', shell=True)

    def execute_command(self, command, shell=False):
        self.logger.debug('Executing command: ' + str(command))
        try:
            process = Popen(command, shell=shell, stdout=PIPE, stderr=PIPE)
        except Exception as e:
            self.logger.debug('Error while executing command: "' + str(command) + '\n' + str(e))
        else:
            stdout, stderr = process.communicate()
            if stdout:
                self.logger.debug('Execution result: ' + stdout)
                return stdout