#Copyright (c) 2015 3D Control Systems LTD

#3DPrinterOS client is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#3DPrinterOS client is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with 3DPrinterOS client.  If not, see <http://www.gnu.org/licenses/>.

# Author: Vladimir Avdeev <another.vic@yandex.ru>, Oleg Panasevych <panasevychol@gmail.com>

import os
import json
import paths
import logging
import marshal

import http_client
from checker_waiter import CheckerWaiter

storage_path = os.path.join(paths.CURRENT_SETTINGS_FOLDER, 'stuff.bin')

def read_login():
    logger = logging.getLogger(__name__)
    try:
        with open(storage_path, "rb") as f:
            marshalled_data = f.read()
    except (IOError, OSError) as e:
        logger.info("Can't read login storage file: " + str(e))
    else:
        try:
            login_password = marshal.loads(marshalled_data)
        except:
            logger.info("Can't decode login and password")
        else:
            return login_password

#login storage with some rational paranoia
def write_login(login, password):
    logger = logging.getLogger(__name__)
    marshalled_data = marshal.dumps([login, password])
    new_storage_file = os.path.join(paths.CURRENT_SETTINGS_FOLDER, 'new_stuff.bin')
    try:
        with open(new_storage_file, "wb") as f:
            f.write(marshalled_data)
    except (IOError, OSError) as e:
        logger.warning('Error writing login to storage file: ' + str(e))
    else:
        try:
            os.remove(storage_path)
        except OSError:
            pass
        try:
            os.rename(new_storage_file, storage_path)
            return True
        except (IOError, OSError) as e:
            logger.warning('Error renaming login to storage file: ' + str(e))
        try:
            os.rename(new_storage_file, storage_path)
            return True
        except (IOError, OSError) as e:
            logger.warning('Error renaming login to storage file even after removal of some logs: ' + str(e))

def logout():
    logger = logging.getLogger(__name__)
    logger.info("Removing login file")
    try:
        os.remove(storage_path)
    except:
        logger.info("Login file removed")

class UserLogin(CheckerWaiter):

    NAME_FOR_LOGGING = 'User login'

    def __init__(self, app):
        self.app = app
        self.login = None
        self.user_token = None
        self.http_client = http_client.HTTPClient(self.app)
        CheckerWaiter.__init__(self, app, self.check_if_login_complete)
        if not app.stop_flag:
            login_password = read_login()
            if login_password:
                error = self.login_as_user(login_password[0], login_password[1], save_password_flag=False)
                if error:
                    self.logger.info(str(error))

    def check_if_login_complete(self):
        return bool(self.user_token)

    def login_as_user(self, login, password, auth_token=None, save_password_flag=True):
        if not login:
            return 0, "Empty login"
        if not password and not auth_token:
            return 0, "Empty password"
        answer = self.http_client.pack_and_send('user_login', login, password, auth_token=auth_token)
        if auth_token:
            save_password_flag = False
        if not answer:
            return 0, "No connection to server"
        else:
            user_token = answer.get('user_token')
            profiles_str = answer.get('all_profiles')
            error = answer.get('error')
            if error:
                self.logger.warning("Error processing user_login " + str(error))
                self.logger.error("Login rejected")
                return error['code'], error['message']
            if login:
                self.login = login # for web_interface to display
                if save_password_flag:
                    if not write_login(login, password):
                        self.logger.warning("Error saving login and password")
            else:
                login = answer.get('user_login')
                if not login:
                    login = "Temporary login"
                self.login = login
            try:
                profiles = json.loads(profiles_str)
            except Exception as e:
                self.logger.warning("Error while parsing profiles: " + str(e))
                return 42, "Error parsing profiles"
            else:
                self.user_token = user_token
                self.profiles = profiles
                self.logger.info("Successful login from user " + login)