#Copyright (c) 2015 3D Control Systems LTD

#3DPrinterOS client is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#3DPrinterOS client is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.

#You should have received a copy of the GNU Affero General Public License
#along with 3DPrinterOS client.  If not, see <http://www.gnu.org/licenses/>.

# Author: Oleg Panasevych <panasevychol@gmail.com>, Vladimir Avdeev <another.vic@yandex.ru>

import os
import logging
import threading
import BaseHTTPServer
from SocketServer import ThreadingMixIn

import config
import page_former
import web_interface_handler


class ThreadedHTTPServer(ThreadingMixIn, BaseHTTPServer.HTTPServer):
    """ This class allows to handle requests in separated threads.
        No further content needed, don't touch this. """


class WebInterface(threading.Thread):

    @staticmethod
    def get_web_content_dir_path():
        working_dir = os.path.dirname(os.path.abspath(__file__))
        return os.path.join(working_dir, 'web_content_files')

    def __init__(self, app):
        self.logger = logging.getLogger(__name__)
        self.app = app
        self.server = None
        threading.Thread.__init__(self, name="WebInterface")

    def find_printer_interface(self, usb_info_string):
        for printer_interface in self.server.app.printer_interfaces:
            if str(printer_interface.usb_info) == usb_info_string:
                return printer_interface

    def get_web_content_files(self, dir):
        web_content_files = {}
        for name in os.listdir(dir):
            file_path = os.path.join(dir, name)
            try:
                if os.path.isdir(file_path):
                    content = self.get_web_content_files(file_path)
                elif name.endswith('.jpg'):
                    with open(file_path, 'rb') as f:
                        content = f.read()
                else:
                    with open(file_path) as f:
                        content = f.read()
            except Exception as e:
                self.logger.error('Error while reading file "' + name + '": ' + str(e))
            else:
                web_content_files[name] = content
        return web_content_files

    def run(self):
        self.logger.info("Starting web server...")
        if config.get_settings()['remote_control']['web_server']:
            IP = config.REMOTE_IP
        else:
            IP = config.LOCAL_IP
        port = config.get_settings()['web_interface']['port']
        try:
            self.server = ThreadedHTTPServer((IP, port), web_interface_handler.WebInterfaceHandler)
        except Exception as e:
            self.logger.error(e)
        else:
            self.logger.info("...web server started")
            self.server.app = self.app
            self.server.web_content_files = self.get_web_content_files(self.get_web_content_dir_path())
            self.server.page_former = page_former.PageFormer(self.server)
            self.server.find_printer_interface = self.find_printer_interface
            self.server.serve_forever()
            self.server.server_close()
            self.logger.info("Web server stop.")